@@ -1,4 +1,37 @@
 <?php
+/**
+ * @author Arthur Schiwon <blizzz@owncloud.com>
+ * @author Bart Visscher <bartv@thisnet.nl>
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Florian Pritz <bluewind@xinu.at>
+ * @author Frank Karlitschek <frank@owncloud.org>
+ * @author Joas Schilling <nickvergessen@owncloud.com>
+ * @author Jörn Friedrich Dreyer <jfd@butonic.de>
+ * @author Lukas Reschke <lukas@owncloud.com>
+ * @author Luke Policinski <lpolicinski@gmail.com>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Roman Geber <rgeber@owncloudapps.com>
+ * @author TheSFReader <TheSFReader@gmail.com>
+ * @author Thomas Müller <thomas.mueller@tmit.eu>
+ * @author Vincent Petry <pvince81@owncloud.com>
+ *
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
+ *
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *
+ */
+\OC::$server->getSession()->close();
 
 // Firefox and Konqueror tries to download application/json for me.  --Arthur
 OCP\JSON::setContentTypeHeader('text/plain');
@@ -7,29 +40,36 @@ OCP\JSON::setContentTypeHeader('text/plain');
 // If not, check the login.
 // If no token is sent along, rely on login only
 
-$allowedPermissions = OCP\PERMISSION_ALL;
+$allowedPermissions = \OCP\Constants::PERMISSION_ALL;
+$errorCode = null;
 
-$l = OC_L10N::get('files');
+$l = \OC::$server->getL10N('files');
 if (empty($_POST['dirToken'])) {
 	// The standard case, files are uploaded through logged in users :)
 	OCP\JSON::checkLoggedIn();
-	$dir = isset($_POST['dir']) ? $_POST['dir'] : "";
+	$dir = isset($_POST['dir']) ? (string)$_POST['dir'] : '';
 	if (!$dir || empty($dir) || $dir === false) {
 		OCP\JSON::error(array('data' => array_merge(array('message' => $l->t('Unable to set upload directory.')))));
 		die();
 	}
 } else {
-	// return only read permissions for public upload
-	$allowedPermissions = OCP\PERMISSION_READ;
-	$public_directory = !empty($_POST['subdir']) ? $_POST['subdir'] : '/';
+	// TODO: ideally this code should be in files_sharing/ajax/upload.php
+	// and the upload/file transfer code needs to be refactored into a utility method
+	// that could be used there
 
-	$linkItem = OCP\Share::getShareByToken($_POST['dirToken']);
+	\OC_User::setIncognitoMode(true);
+
+	// return only read permissions for public upload
+	$allowedPermissions = \OCP\Constants::PERMISSION_READ;
+	$publicDirectory = !empty($_POST['subdir']) ? (string)$_POST['subdir'] : '/';
+
+	$linkItem = OCP\Share::getShareByToken((string)$_POST['dirToken']);
 	if ($linkItem === false) {
 		OCP\JSON::error(array('data' => array_merge(array('message' => $l->t('Invalid Token')))));
 		die();
 	}
 
-	if (!($linkItem['permissions'] & OCP\PERMISSION_CREATE)) {
+	if (!($linkItem['permissions'] & \OCP\Constants::PERMISSION_CREATE)) {
 		OCP\JSON::checkLoggedIn();
 	} else {
 		// resolve reshares
@@ -42,23 +82,27 @@ if (empty($_POST['dirToken'])) {
 
 		// The token defines the target directory (security reasons)
 		$path = \OC\Files\Filesystem::getPath($linkItem['file_source']);
+		if($path === null) {
+			OCP\JSON::error(array('data' => array_merge(array('message' => $l->t('Unable to set upload directory.')))));
+			die();
+		}
 		$dir = sprintf(
 			"/%s/%s",
 			$path,
-			$public_directory
+			$publicDirectory
 		);
 
 		if (!$dir || empty($dir) || $dir === false) {
 			OCP\JSON::error(array('data' => array_merge(array('message' => $l->t('Unable to set upload directory.')))));
 			die();
 		}
+
+		$dir = rtrim($dir, '/');
 	}
 }
 
-
 OCP\JSON::callCheck();
 
-
 // get array with current storage stats (e.g. max file size)
 $storageStats = \OCA\Files\Helper::buildFileStorageStatistics($dir);
 
@@ -80,7 +124,7 @@ foreach ($_FILES['files']['error'] as $error) {
 			UPLOAD_ERR_CANT_WRITE => $l->t('Failed to write to disk'),
 		);
 		$errorMessage = $errors[$error];
-		\OCP\Util::writeLog('files', "Upload error: $error - $errorMessage", \OCP\Util::ERROR);
+		\OC::$server->getLogger()->alert("Upload error: $error - $errorMessage", array('app' => 'files'));
 		OCP\JSON::error(array('data' => array_merge(array('message' => $errorMessage), $storageStats)));
 		exit();
 	}
@@ -107,24 +151,43 @@ $result = array();
 if (strpos($dir, '..') === false) {
 	$fileCount = count($files['name']);
 	for ($i = 0; $i < $fileCount; $i++) {
-		// $path needs to be normalized - this failed within drag'n'drop upload to a sub-folder
-		if (isset($_POST['resolution']) && $_POST['resolution']==='autorename') {
-			// append a number in brackets like 'filename (2).ext'
-			$target = OCP\Files::buildNotExistingFileName(stripslashes($dir), $files['name'][$i]);
+
+		if (isset($_POST['resolution'])) {
+			$resolution = $_POST['resolution'];
 		} else {
-			$target = \OC\Files\Filesystem::normalizePath(stripslashes($dir).'/'.$files['name'][$i]);
+			$resolution = null;
 		}
 
-		$directory = \OC\Files\Filesystem::normalizePath(stripslashes($dir));
-		if (isset($public_directory)) {
-			// If we are uploading from the public app,
-			// we want to send the relative path in the ajax request.
-			$directory = $public_directory;
+		// target directory for when uploading folders
+		$relativePath = '';
+		if(!empty($_POST['file_directory'])) {
+			$relativePath = '/'.$_POST['file_directory'];
 		}
 
-		if ( ! \OC\Files\Filesystem::file_exists($target)
-			|| (isset($_POST['resolution']) && $_POST['resolution']==='replace')
-		) {
+		// $path needs to be normalized - this failed within drag'n'drop upload to a sub-folder
+		if ($resolution === 'autorename') {
+			// append a number in brackets like 'filename (2).ext'
+			$target = OCP\Files::buildNotExistingFileName($dir . $relativePath, $files['name'][$i]);
+		} else {
+			$target = \OC\Files\Filesystem::normalizePath($dir . $relativePath.'/'.$files['name'][$i]);
+		}
+
+		// relative dir to return to the client
+		if (isset($publicDirectory)) {
+			// path relative to the public root
+			$returnedDir = $publicDirectory . $relativePath;
+		} else {
+			// full path
+			$returnedDir = $dir . $relativePath;
+		}
+		$returnedDir = \OC\Files\Filesystem::normalizePath($returnedDir);
+
+
+		$exists = \OC\Files\Filesystem::file_exists($target);
+		if ($exists) {
+			$updatable = \OC\Files\Filesystem::isUpdatable($target);
+		}
+		if ( ! $exists || ($updatable && $resolution === 'replace' ) ) {
 			// upload and overwrite file
 			try
 			{
@@ -135,21 +198,17 @@ if (strpos($dir, '..') === false) {
 
 					$meta = \OC\Files\Filesystem::getFileInfo($target);
 					if ($meta === false) {
-						$error = $l->t('Upload failed. Could not get file info.');
+						$error = $l->t('The target folder has been moved or deleted.');
+						$errorCode = 'targetnotfound';
 					} else {
-						$result[] = array('status' => 'success',
-							'mime' => $meta['mimetype'],
-							'mtime' => $meta['mtime'],
-							'size' => $meta['size'],
-							'id' => $meta['fileid'],
-							'name' => basename($target),
-							'etag' => $meta['etag'],
-							'originalname' => $files['tmp_name'][$i],
-							'uploadMaxFilesize' => $maxUploadFileSize,
-							'maxHumanFilesize' => $maxHumanFileSize,
-							'permissions' => $meta['permissions'] & $allowedPermissions,
-							'directory' => $directory,
-						);
+						$data = \OCA\Files\Helper::formatFileInfo($meta);
+						$data['status'] = 'success';
+						$data['originalname'] = $files['name'][$i];
+						$data['uploadMaxFilesize'] = $maxUploadFileSize;
+						$data['maxHumanFilesize'] = $maxHumanFileSize;
+						$data['permissions'] = $meta['permissions'] & $allowedPermissions;
+						$data['directory'] = $returnedDir;
+						$result[] = $data;
 					}
 
 				} else {
@@ -158,26 +217,25 @@ if (strpos($dir, '..') === false) {
 			} catch(Exception $ex) {
 				$error = $ex->getMessage();
 			}
-			
+
 		} else {
 			// file already exists
 			$meta = \OC\Files\Filesystem::getFileInfo($target);
 			if ($meta === false) {
 				$error = $l->t('Upload failed. Could not get file info.');
 			} else {
-				$result[] = array('status' => 'existserror',
-					'mime' => $meta['mimetype'],
-					'mtime' => $meta['mtime'],
-					'size' => $meta['size'],
-					'id' => $meta['fileid'],
-					'name' => basename($target),
-					'etag' => $meta['etag'],
-					'originalname' => $files['tmp_name'][$i],
-					'uploadMaxFilesize' => $maxUploadFileSize,
-					'maxHumanFilesize' => $maxHumanFileSize,
-					'permissions' => $meta['permissions'] & $allowedPermissions,
-					'directory' => $directory,
-				);
+				$data = \OCA\Files\Helper::formatFileInfo($meta);
+				if ($updatable) {
+					$data['status'] = 'existserror';
+				} else {
+					$data['status'] = 'readonly';
+				}
+				$data['originalname'] = $files['name'][$i];
+				$data['uploadMaxFilesize'] = $maxUploadFileSize;
+				$data['maxHumanFilesize'] = $maxHumanFileSize;
+				$data['permissions'] = $meta['permissions'] & $allowedPermissions;
+				$data['directory'] = $returnedDir;
+				$result[] = $data;
 			}
 		}
 	}
@@ -187,7 +245,6 @@ if (strpos($dir, '..') === false) {
 
 if ($error === false) {
 	OCP\JSON::encodedPrint($result);
-	exit();
 } else {
-	OCP\JSON::error(array(array('data' => array_merge(array('message' => $error), $storageStats))));
+	OCP\JSON::error(array(array('data' => array_merge(array('message' => $error, 'code' => $errorCode), $storageStats))));
 }
