@@ -2,8 +2,24 @@
 
 <header>
 	<div id="header">
-		<a href="<?php print_unescaped(link_to('', 'index.php')); ?>" title="" id="owncloud">
-			<img class="svg" src="<?php print_unescaped(image_path('', 'logo-wide.svg')); ?>" alt="<?php p($theme->getName()); ?>" /></a>
+		<a href="<?php print_unescaped(link_to('', 'index.php')); ?>"
+		   title="" id="owncloud">
+			<div class="logo-icon svg">
+			</div>
+		</a>
+
+		<div class="header-appname-container">
+			<h1 class="header-appname">
+				<?php
+					if(OC_Util::getEditionString() === '') {
+						p($theme->getName());
+					} else {
+						print_unescaped($theme->getHTMLName());
+					}
+				?>
+			</h1>
+		</div>
+
 		<div id="logo-claim" style="display:none;"><?php p($theme->getLogoClaim()); ?></div>
 		<div class="header-right">
 			<span id="details"><?php p($l->t('shared by %s', $_['displayName'])) ?></span>
@@ -21,7 +37,7 @@
 		</div>
 	</div>
 
-	<div id='gallery' class="hascontrols" data-token="<?php isset($_['token']) ? p($_['token']) : p(false) ?>"></div>
+	<div id='gallery' class="hascontrols" data-requesttoken="<?php p($_['requesttoken'])?>" data-token="<?php isset($_['token']) ? p($_['token']) : p(false) ?>"></div>
 </div>
 
 	<div class="push"></div><!-- for sticky footer -->
