@@ -19,15 +19,18 @@
  *
  */
 
+/* global alert */
+
 /**
  * this class to ease the usage of jquery dialogs
+ * @lends OC.dialogs
  */
 var OCdialogs = {
 	// dialog button types
 	YES_NO_BUTTONS:		70,
 	OK_BUTTONS:		71,
 	// used to name each dialog
-	dialogs_counter: 0,
+	dialogsCounter: 0,
 	/**
 	* displays alert dialog
 	* @param text content of dialog
@@ -36,7 +39,14 @@ var OCdialogs = {
 	* @param modal make the dialog modal
 	*/
 	alert:function(text, title, callback, modal) {
-		this.message(text, title, 'alert', OCdialogs.OK_BUTTON, callback, modal);
+		this.message(
+			text,
+			title,
+			'alert',
+			OCdialogs.OK_BUTTON,
+			callback,
+			modal
+		);
 	},
 	/**
 	* displays info dialog
@@ -52,32 +62,102 @@ var OCdialogs = {
 	* displays confirmation dialog
 	* @param text content of dialog
 	* @param title dialog title
-	* @param callback which will be triggered when user presses YES or NO (true or false would be passed to callback respectively)
+	* @param callback which will be triggered when user presses YES or NO
+	*        (true or false would be passed to callback respectively)
 	* @param modal make the dialog modal
 	*/
 	confirm:function(text, title, callback, modal) {
-		this.message(text, title, 'notice', OCdialogs.YES_NO_BUTTONS, callback, modal);
+		return this.message(
+			text,
+			title,
+			'notice',
+			OCdialogs.YES_NO_BUTTONS,
+			callback,
+			modal
+		);
+	},
+	/**
+	 * displays prompt dialog
+	 * @param text content of dialog
+	 * @param title dialog title
+	 * @param callback which will be triggered when user presses YES or NO
+	 *        (true or false would be passed to callback respectively)
+	 * @param modal make the dialog modal
+	 * @param name name of the input field
+	 * @param password whether the input should be a password input
+	 */
+	prompt: function (text, title, callback, modal, name, password) {
+		return $.when(this._getMessageTemplate()).then(function ($tmpl) {
+			var dialogName = 'oc-dialog-' + OCdialogs.dialogsCounter + '-content';
+			var dialogId = '#' + dialogName;
+			var $dlg = $tmpl.octemplate({
+				dialog_name: dialogName,
+				title      : title,
+				message    : text,
+				type       : 'notice'
+			});
+			var input = $('<input/>');
+			input.attr('type', password ? 'password' : 'text').attr('id', dialogName + '-input');
+			var label = $('<label/>').attr('for', dialogName + '-input').text(name + ': ');
+			$dlg.append(label);
+			$dlg.append(input);
+			if (modal === undefined) {
+				modal = false;
+			}
+			$('body').append($dlg);
+			var buttonlist = [{
+					text : t('core', 'No'),
+					click: function () {
+						if (callback !== undefined) {
+							callback(false, input.val());
+						}
+						$(dialogId).ocdialog('close');
+					}
+				}, {
+					text         : t('core', 'Yes'),
+					click        : function () {
+						if (callback !== undefined) {
+							callback(true, input.val());
+						}
+						$(dialogId).ocdialog('close');
+					},
+					defaultButton: true
+				}
+			];
+
+			$(dialogId).ocdialog({
+				closeOnEscape: true,
+				modal        : modal,
+				buttons      : buttonlist
+			});
+			OCdialogs.dialogsCounter++;
+		});
 	},
 	/**
 	 * show a file picker to pick a file from
 	 * @param title dialog title
 	 * @param callback which will be triggered when user presses Choose
 	 * @param multiselect whether it should be possible to select multiple files
-	 * @param mimetype_filter mimetype to filter by
+	 * @param mimetypeFilter mimetype to filter by - directories will always be included
 	 * @param modal make the dialog modal
 	*/
-	filepicker:function(title, callback, multiselect, mimetype_filter, modal) {
+	filepicker:function(title, callback, multiselect, mimetypeFilter, modal) {
 		var self = this;
+		// avoid opening the picker twice
+		if (this.filepicker.loading) {
+			return;
+		}
+		this.filepicker.loading = true;
 		$.when(this._getFilePickerTemplate()).then(function($tmpl) {
-			var dialog_name = 'oc-dialog-filepicker-content';
-			var dialog_id = '#' + dialog_name;
+			self.filepicker.loading = false;
+			var dialogName = 'oc-dialog-filepicker-content';
 			if(self.$filePicker) {
 				self.$filePicker.ocdialog('close');
 			}
 			self.$filePicker = $tmpl.octemplate({
-				dialog_name: dialog_name,
+				dialog_name: dialogName,
 				title: title
-			}).data('path', '').data('multiselect', multiselect).data('mimetype', mimetype_filter);
+			}).data('path', '').data('multiselect', multiselect).data('mimetype', mimetypeFilter);
 
 			if (modal === undefined) {
 				modal = false;
@@ -85,8 +165,8 @@ var OCdialogs = {
 			if (multiselect === undefined) {
 				multiselect = false;
 			}
-			if (mimetype_filter === undefined) {
-				mimetype_filter = '';
+			if (mimetypeFilter === undefined) {
+				mimetypeFilter = '';
 			}
 
 			$('body').append(self.$filePicker);
@@ -127,21 +207,26 @@ var OCdialogs = {
 
 			self.$filePicker.ocdialog({
 				closeOnEscape: true,
-				width: (4/9)*$(document).width(),
+				// max-width of 600
+				width: Math.min((4/5)*$(document).width(), 600),
 				height: 420,
 				modal: modal,
 				buttons: buttonlist,
-				close: function(event, ui) {
+				close: function() {
 					try {
 						$(this).ocdialog('destroy').remove();
 					} catch(e) {}
 					self.$filePicker = null;
 				}
 			});
+			if (!OC.Util.hasSVGSupport()) {
+				OC.Util.replaceSVG(self.$filePicker.parent());
+			}
 		})
 		.fail(function(status, error) {
 			// If the method is called while navigating away
 			// from the page, it is probably not needed ;)
+			self.filepicker.loading = false;
 			if(status !== 0) {
 				alert(t('core', 'Error loading file picker template: {error}', {error: error}));
 			}
@@ -151,15 +236,15 @@ var OCdialogs = {
 	 * Displays raw dialog
 	 * You better use a wrapper instead ...
 	*/
-	message:function(content, title, dialog_type, buttons, callback, modal) {
-		$.when(this._getMessageTemplate()).then(function($tmpl) {
-			var dialog_name = 'oc-dialog-' + OCdialogs.dialogs_counter + '-content';
-			var dialog_id = '#' + dialog_name;
+	message:function(content, title, dialogType, buttons, callback, modal) {
+		return $.when(this._getMessageTemplate()).then(function($tmpl) {
+			var dialogName = 'oc-dialog-' + OCdialogs.dialogsCounter + '-content';
+			var dialogId = '#' + dialogName;
 			var $dlg = $tmpl.octemplate({
-				dialog_name: dialog_name,
+				dialog_name: dialogName,
 				title: title,
 				message: content,
-				type: dialog_type
+				type: dialogType
 			});
 			if (modal === undefined) {
 				modal = false;
@@ -167,48 +252,48 @@ var OCdialogs = {
 			$('body').append($dlg);
 			var buttonlist = [];
 			switch (buttons) {
-				case OCdialogs.YES_NO_BUTTONS:
-					buttonlist = [{
-						text: t('core', 'Yes'),
-						click: function(){
-							if (callback !== undefined) {
-								callback(true);
-							}
-							$(dialog_id).ocdialog('close');
-						},
-						defaultButton: true
+			case OCdialogs.YES_NO_BUTTONS:
+				buttonlist = [{
+					text: t('core', 'No'),
+					click: function(){
+						if (callback !== undefined) {
+							callback(false);
+						}
+						$(dialogId).ocdialog('close');
+					}
+				},
+				{
+					text: t('core', 'Yes'),
+					click: function(){
+						if (callback !== undefined) {
+							callback(true);
+						}
+						$(dialogId).ocdialog('close');
 					},
-					{
-						text: t('core', 'No'),
-						click: function(){
-							if (callback !== undefined) {
-								callback(false);
-							}
-							$(dialog_id).ocdialog('close');
-						}
-					}];
+					defaultButton: true
+				}];
 				break;
-				case OCdialogs.OK_BUTTON:
-					var functionToCall = function() {
-						$(dialog_id).ocdialog('close');
-						if(callback !== undefined) {
-							callback();
-						}
-					};
-					buttonlist[0] = {
-						text: t('core', 'Ok'),
-						click: functionToCall,
-						defaultButton: true
-					};
+			case OCdialogs.OK_BUTTON:
+				var functionToCall = function() {
+					$(dialogId).ocdialog('close');
+					if(callback !== undefined) {
+						callback();
+					}
+				};
+				buttonlist[0] = {
+					text: t('core', 'Ok'),
+					click: functionToCall,
+					defaultButton: true
+				};
 				break;
 			}
 
-			$(dialog_id).ocdialog({
+			$(dialogId).ocdialog({
 				closeOnEscape: true,
 				modal: modal,
 				buttons: buttonlist
 			});
-			OCdialogs.dialogs_counter++;
+			OCdialogs.dialogsCounter++;
 		})
 		.fail(function(status, error) {
 			// If the method is called while navigating away from
@@ -246,7 +331,7 @@ var OCdialogs = {
 					image.onload = function () {
 						var url = crop(image);
 						deferred.resolve(url);
-					}
+					};
 				};
 				reader.readAsArrayBuffer(file);
 			} else {
@@ -279,187 +364,209 @@ var OCdialogs = {
 			return canvas.toDataURL("image/png", 0.7);
 		};
 
-		var addConflict = function(conflicts, original, replacement) {
+		var addConflict = function($conflicts, original, replacement) {
 
-			var conflict = conflicts.find('.template').clone().removeClass('template').addClass('conflict');
+			var $conflict = $conflicts.find('.template').clone().removeClass('template').addClass('conflict');
+			var $originalDiv = $conflict.find('.original');
+			var $replacementDiv = $conflict.find('.replacement');
 
-			conflict.data('data',data);
+			$conflict.data('data',data);
 
-			conflict.find('.filename').text(original.name);
-			conflict.find('.original .size').text(humanFileSize(original.size));
-			conflict.find('.original .mtime').text(formatDate(original.mtime*1000));
+			$conflict.find('.filename').text(original.name);
+			$originalDiv.find('.size').text(humanFileSize(original.size));
+			$originalDiv.find('.mtime').text(formatDate(original.mtime));
 			// ie sucks
 			if (replacement.size && replacement.lastModifiedDate) {
-				conflict.find('.replacement .size').text(humanFileSize(replacement.size));
-				conflict.find('.replacement .mtime').text(formatDate(replacement.lastModifiedDate));
+				$replacementDiv.find('.size').text(humanFileSize(replacement.size));
+				$replacementDiv.find('.mtime').text(formatDate(replacement.lastModifiedDate));
 			}
-			var path = getPathForPreview(original.name);
-			Files.lazyLoadPreview(path, original.mime, function(previewpath){
-				conflict.find('.original .icon').css('background-image','url('+previewpath+')');
-			}, 96, 96, original.etag);
+			var path = original.directory + '/' +original.name;
+			var urlSpec = {
+				file:		path,
+				x:		96,
+				y:		96,
+				c:		original.etag,
+				forceIcon:	0
+			};
+			var previewpath = OC.generateUrl('/core/preview.png?') + $.param(urlSpec);
+			// Escaping single quotes
+			previewpath = previewpath.replace(/'/g, "%27")
+			$originalDiv.find('.icon').css({"background-image":   "url('" + previewpath + "')"});
 			getCroppedPreview(replacement).then(
 				function(path){
-					conflict.find('.replacement .icon').css('background-image','url(' + path + ')');
+					$replacementDiv.find('.icon').css('background-image','url(' + path + ')');
 				}, function(){
 					Files.getMimeIcon(replacement.type,function(path){
-						conflict.find('.replacement .icon').css('background-image','url(' + path + ')');
+						$replacementDiv.find('.icon').css('background-image','url(' + path + ')');
 					});
 				}
 			);
-			conflicts.append(conflict);
+			$conflicts.append($conflict);
 
 			//set more recent mtime bold
 			// ie sucks
-			if (replacement.lastModifiedDate && replacement.lastModifiedDate.getTime() > original.mtime*1000) {
-				conflict.find('.replacement .mtime').css('font-weight', 'bold');
-			} else if (replacement.lastModifiedDate && replacement.lastModifiedDate.getTime() < original.mtime*1000) {
-				conflict.find('.original .mtime').css('font-weight', 'bold');
+			if (replacement.lastModifiedDate && replacement.lastModifiedDate.getTime() > original.mtime) {
+				$replacementDiv.find('.mtime').css('font-weight', 'bold');
+			} else if (replacement.lastModifiedDate && replacement.lastModifiedDate.getTime() < original.mtime) {
+				$originalDiv.find('.mtime').css('font-weight', 'bold');
 			} else {
 				//TODO add to same mtime collection?
 			}
 
 			// set bigger size bold
 			if (replacement.size && replacement.size > original.size) {
-				conflict.find('.replacement .size').css('font-weight', 'bold');
+				$replacementDiv.find('.size').css('font-weight', 'bold');
 			} else if (replacement.size && replacement.size < original.size) {
-				conflict.find('.original .size').css('font-weight', 'bold');
+				$originalDiv.find('.size').css('font-weight', 'bold');
 			} else {
 				//TODO add to same size collection?
 			}
 
 			//TODO show skip action for files with same size and mtime in bottom row
 
+			// always keep readonly files
+
+			if (original.status === 'readonly') {
+				$originalDiv
+					.addClass('readonly')
+					.find('input[type="checkbox"]')
+						.prop('checked', true)
+						.prop('disabled', true);
+				$originalDiv.find('.message')
+					.text(t('core','read-only'))
+			}
 		};
 		//var selection = controller.getSelection(data.originalFiles);
 		//if (selection.defaultAction) {
 		//	controller[selection.defaultAction](data);
 		//} else {
-			var dialog_name = 'oc-dialog-fileexists-content';
-			var dialog_id = '#' + dialog_name;
-			if (this._fileexistsshown) {
-				// add conflict
+		var dialogName = 'oc-dialog-fileexists-content';
+		var dialogId = '#' + dialogName;
+		if (this._fileexistsshown) {
+			// add conflict
 
-				var conflicts = $(dialog_id+ ' .conflicts');
-				addConflict(conflicts, original, replacement);
+			var $conflicts = $(dialogId+ ' .conflicts');
+			addConflict($conflicts, original, replacement);
 
-				var count = $(dialog_id+ ' .conflict').length;
-				var title = n('files',
-								'{count} file conflict',
-								'{count} file conflicts',
-								count,
-								{count:count}
-							);
-				$(dialog_id).parent().children('.oc-dialog-title').text(title);
+			var count = $(dialogId+ ' .conflict').length;
+			var title = n('core',
+							'{count} file conflict',
+							'{count} file conflicts',
+							count,
+							{count:count}
+						);
+			$(dialogId).parent().children('.oc-dialog-title').text(title);
 
-				//recalculate dimensions
-				$(window).trigger('resize');
+			//recalculate dimensions
+			$(window).trigger('resize');
 
-			} else {
-				//create dialog
-				this._fileexistsshown = true;
-				$.when(this._getFileExistsTemplate()).then(function($tmpl) {
-					var title = t('core','One file conflict');
-					var $dlg = $tmpl.octemplate({
-						dialog_name: dialog_name,
-						title: title,
-						type: 'fileexists',
+		} else {
+			//create dialog
+			this._fileexistsshown = true;
+			$.when(this._getFileExistsTemplate()).then(function($tmpl) {
+				var title = t('core','One file conflict');
+				var $dlg = $tmpl.octemplate({
+					dialog_name: dialogName,
+					title: title,
+					type: 'fileexists',
 
-						allnewfiles: t('core','New Files'),
-						allexistingfiles: t('core','Already existing files'),
+					allnewfiles: t('core','New Files'),
+					allexistingfiles: t('core','Already existing files'),
 
-						why: t('core','Which files do you want to keep?'),
-						what: t('core','If you select both versions, the copied file will have a number added to its name.')
-					});
-					$('body').append($dlg);
+					why: t('core','Which files do you want to keep?'),
+					what: t('core','If you select both versions, the copied file will have a number added to its name.')
+				});
+				$('body').append($dlg);
 
-					var conflicts = $($dlg).find('.conflicts');
-					addConflict(conflicts, original, replacement);
+				var $conflicts = $dlg.find('.conflicts');
+				addConflict($conflicts, original, replacement);
 
-					buttonlist = [{
-							text: t('core', 'Cancel'),
-							classes: 'cancel',
-							click: function(){
-								if ( typeof controller.onCancel !== 'undefined') {
-									controller.onCancel(data);
-								}
-								$(dialog_id).ocdialog('close');
+				var buttonlist = [{
+						text: t('core', 'Cancel'),
+						classes: 'cancel',
+						click: function(){
+							if ( typeof controller.onCancel !== 'undefined') {
+								controller.onCancel(data);
 							}
-						},
-						{
-							text: t('core', 'Continue'),
-							classes: 'continue',
-							click: function(){
-								if ( typeof controller.onContinue !== 'undefined') {
-									controller.onContinue($(dialog_id + ' .conflict'));
-								}
-								$(dialog_id).ocdialog('close');
+							$(dialogId).ocdialog('close');
+						}
+					},
+					{
+						text: t('core', 'Continue'),
+						classes: 'continue',
+						click: function(){
+							if ( typeof controller.onContinue !== 'undefined') {
+								controller.onContinue($(dialogId + ' .conflict'));
 							}
-						}];
+							$(dialogId).ocdialog('close');
+						}
+					}];
 
-					$(dialog_id).ocdialog({
-						width: 500,
-						closeOnEscape: true,
-						modal: true,
-						buttons: buttonlist,
-						closeButton: null,
-						close: function(event, ui) {
-								self._fileexistsshown = false;
+				$(dialogId).ocdialog({
+					width: 500,
+					closeOnEscape: true,
+					modal: true,
+					buttons: buttonlist,
+					closeButton: null,
+					close: function() {
+							self._fileexistsshown = false;
 							$(this).ocdialog('destroy').remove();
 						}
-					});
-
-					$(dialog_id).css('height','auto');
-
-					//add checkbox toggling actions
-					$(dialog_id).find('.allnewfiles').on('click', function() {
-						var checkboxes = $(dialog_id).find('.conflict .replacement input[type="checkbox"]');
-						checkboxes.prop('checked', $(this).prop('checked'));
-					});
-					$(dialog_id).find('.allexistingfiles').on('click', function() {
-						var checkboxes = $(dialog_id).find('.conflict .original input[type="checkbox"]');
-						checkboxes.prop('checked', $(this).prop('checked'));
-					});
-					$(dialog_id).find('.conflicts').on('click', '.replacement,.original', function() {
-						var checkbox = $(this).find('input[type="checkbox"]');
-						checkbox.prop('checked', !checkbox.prop('checked'));
-					});
-					$(dialog_id).find('.conflicts').on('click', 'input[type="checkbox"]', function() {
-						var checkbox = $(this);
-						checkbox.prop('checked', !checkbox.prop('checked'));
-					});
-
-					//update counters
-					$(dialog_id).on('click', '.replacement,.allnewfiles', function() {
-						var count = $(dialog_id).find('.conflict .replacement input[type="checkbox"]:checked').length;
-						if (count === $(dialog_id+ ' .conflict').length) {
-							$(dialog_id).find('.allnewfiles').prop('checked', true);
-							$(dialog_id).find('.allnewfiles + .count').text(t('files','(all selected)'));
-						} else if (count > 0) {
-							$(dialog_id).find('.allnewfiles').prop('checked', false);
-							$(dialog_id).find('.allnewfiles + .count').text(t('files','({count} selected)',{count:count}));
-						} else {
-							$(dialog_id).find('.allnewfiles').prop('checked', false);
-							$(dialog_id).find('.allnewfiles + .count').text('');
-						}
-					});
-					$(dialog_id).on('click', '.original,.allexistingfiles', function(){
-						var count = $(dialog_id).find('.conflict .original input[type="checkbox"]:checked').length;
-						if (count === $(dialog_id+ ' .conflict').length) {
-							$(dialog_id).find('.allexistingfiles').prop('checked', true);
-							$(dialog_id).find('.allexistingfiles + .count').text(t('files','(all selected)'));
-						} else if (count > 0) {
-							$(dialog_id).find('.allexistingfiles').prop('checked', false);
-							$(dialog_id).find('.allexistingfiles + .count').text(t('files','({count} selected)',{count:count}));
-						} else {
-							$(dialog_id).find('.allexistingfiles').prop('checked', false);
-							$(dialog_id).find('.allexistingfiles + .count').text('');
-						}
-					});
-				})
-				.fail(function() {
-					alert(t('core', 'Error loading file exists template'));
 				});
-			}
+
+				$(dialogId).css('height','auto');
+
+				//add checkbox toggling actions
+				$(dialogId).find('.allnewfiles').on('click', function() {
+					var $checkboxes = $(dialogId).find('.conflict .replacement input[type="checkbox"]');
+					$checkboxes.prop('checked', $(this).prop('checked'));
+				});
+				$(dialogId).find('.allexistingfiles').on('click', function() {
+					var $checkboxes = $(dialogId).find('.conflict .original:not(.readonly) input[type="checkbox"]');
+					$checkboxes.prop('checked', $(this).prop('checked'));
+				});
+				$(dialogId).find('.conflicts').on('click', '.replacement,.original:not(.readonly)', function() {
+					var $checkbox = $(this).find('input[type="checkbox"]');
+					$checkbox.prop('checked', !$checkbox.prop('checked'));
+				});
+				$(dialogId).find('.conflicts').on('click', '.replacement input[type="checkbox"],.original:not(.readonly) input[type="checkbox"]', function() {
+					var $checkbox = $(this);
+					$checkbox.prop('checked', !checkbox.prop('checked'));
+				});
+
+				//update counters
+				$(dialogId).on('click', '.replacement,.allnewfiles', function() {
+					var count = $(dialogId).find('.conflict .replacement input[type="checkbox"]:checked').length;
+					if (count === $(dialogId+ ' .conflict').length) {
+						$(dialogId).find('.allnewfiles').prop('checked', true);
+						$(dialogId).find('.allnewfiles + .count').text(t('core','(all selected)'));
+					} else if (count > 0) {
+						$(dialogId).find('.allnewfiles').prop('checked', false);
+						$(dialogId).find('.allnewfiles + .count').text(t('core','({count} selected)',{count:count}));
+					} else {
+						$(dialogId).find('.allnewfiles').prop('checked', false);
+						$(dialogId).find('.allnewfiles + .count').text('');
+					}
+				});
+				$(dialogId).on('click', '.original,.allexistingfiles', function(){
+					var count = $(dialogId).find('.conflict .original input[type="checkbox"]:checked').length;
+					if (count === $(dialogId+ ' .conflict').length) {
+						$(dialogId).find('.allexistingfiles').prop('checked', true);
+						$(dialogId).find('.allexistingfiles + .count').text(t('core','(all selected)'));
+					} else if (count > 0) {
+						$(dialogId).find('.allexistingfiles').prop('checked', false);
+						$(dialogId).find('.allexistingfiles + .count')
+							.text(t('core','({count} selected)',{count:count}));
+					} else {
+						$(dialogId).find('.allexistingfiles').prop('checked', false);
+						$(dialogId).find('.allexistingfiles + .count').text('');
+					}
+				});
+			})
+			.fail(function() {
+				alert(t('core', 'Error loading file exists template'));
+			});
+		}
 		//}
 	},
 	_getFilePickerTemplate: function() {
@@ -517,20 +624,13 @@ var OCdialogs = {
 		}
 
 		return $.getJSON(
-			OC.filePath('files', 'ajax', 'rawlist.php'),
+			OC.filePath('files', 'ajax', 'list.php'),
 			{
 				dir: dir,
 				mimetypes: JSON.stringify(mimeType)
 			}
 		);
 	},
-	_determineValue: function(element) {
-		if ( $(element).attr('type') === 'checkbox' ) {
-			return element.checked;
-		} else {
-			return $(element).val();
-		}
-	},
 
 	/**
 	 * fills the filepicker with files
@@ -542,7 +642,7 @@ var OCdialogs = {
 		this.$filelist.empty().addClass('loading');
 		this.$filePicker.data('path', dir);
 		$.when(this._getFileList(dir, this.$filePicker.data('mimetype'))).then(function(response) {
-			$.each(response.data, function(index, file) {
+			$.each(response.data.files, function(index, file) {
 				if (file.type === 'dir') {
 					dirs.push(file);
 				} else {
@@ -558,13 +658,25 @@ var OCdialogs = {
 					type: entry.type,
 					dir: dir,
 					filename: entry.name,
-					date: OC.mtime2date(entry.mtime)
+					date: OC.Util.relativeModifiedDate(entry.mtime)
 				});
-				$li.find('img').attr('src', entry.mimetype_icon);
+				if (entry.isPreviewAvailable) {
+					var urlSpec = {
+						file: dir + '/' + entry.name
+					};
+					var previewUrl = OC.generateUrl('/core/preview.png?') + $.param(urlSpec);
+					$li.find('img').attr('src', previewUrl);
+				}
+				else {
+					$li.find('img').attr('src', OC.Util.replaceSVGIcon(entry.icon));
+				}
 				self.$filelist.append($li);
 			});
 
 			self.$filelist.removeClass('loading');
+			if (!OC.Util.hasSVGSupport()) {
+				OC.Util.replaceSVG(self.$filePicker.find('.dirtree'));
+			}
 		});
 	},
 	/**
@@ -590,7 +702,7 @@ var OCdialogs = {
 		}
 		$template.octemplate({
 			dir: '',
-			name: '&nbsp;&nbsp;&nbsp;&nbsp;' // Ugly but works ;)
+			name: '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' // Ugly but works ;)
 		}, {escapeFunction: null}).addClass('home svg').prependTo(this.$dirTree);
 	},
 	/**
@@ -610,7 +722,6 @@ var OCdialogs = {
 				this.$filelist.find('.filepicker_element_selected').removeClass('filepicker_element_selected');
 			}
 			$element.toggleClass('filepicker_element_selected');
-			return;
 		} else if ( $element.data('type') === 'dir' ) {
 			this._fillFilePicker(this.$filePicker.data('path') + '/' + $element.data('entryname'));
 		}
