@@ -1,9 +1,25 @@
 <?php
 /**
- * Copyright (c) 2012 Robin Appelman <icewind@owncloud.com>
- * This file is licensed under the Affero General Public License version 3 or
- * later.
- * See the COPYING-README file.
+ * @author Björn Schießle <schiessle@owncloud.com>
+ * @author Morris Jobke <hey@morrisjobke.de>
+ * @author Robin Appelman <icewind@owncloud.com>
+ * @author Robin McCorkell <rmccorkell@karoshi.org.uk>
+ *
+ * @copyright Copyright (c) 2015, ownCloud, Inc.
+ * @license AGPL-3.0
+ *
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License, version 3,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU Affero General Public License for more details.
+ *
+ * You should have received a copy of the GNU Affero General Public License, version 3,
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>
+ *
  */
 
 namespace OC\Files\Mount;
@@ -12,14 +28,14 @@ use \OC\Files\Filesystem;
 
 class Manager {
 	/**
-	 * @var Mount[]
+	 * @var MountPoint[]
 	 */
 	private $mounts = array();
 
 	/**
-	 * @param Mount $mount
+	 * @param MountPoint $mount
 	 */
-	public function addMount($mount) {
+	public function addMount(MountPoint $mount) {
 		$this->mounts[$mount->getMountPoint()] = $mount;
 	}
 
@@ -47,7 +63,7 @@ class Manager {
 	 * Find the mount for $path
 	 *
 	 * @param string $path
-	 * @return Mount
+	 * @return MountPoint
 	 */
 	public function find($path) {
 		\OC_Util::setupFS();
@@ -75,7 +91,7 @@ class Manager {
 	 * Find all mounts in $path
 	 *
 	 * @param string $path
-	 * @return Mount[]
+	 * @return MountPoint[]
 	 */
 	public function findIn($path) {
 		\OC_Util::setupFS();
@@ -99,7 +115,7 @@ class Manager {
 	 * Find mounts by storage id
 	 *
 	 * @param string $id
-	 * @return Mount[]
+	 * @return MountPoint[]
 	 */
 	public function findByStorageId($id) {
 		\OC_Util::setupFS();
@@ -116,7 +132,7 @@ class Manager {
 	}
 
 	/**
-	 * @return Mount[]
+	 * @return MountPoint[]
 	 */
 	public function getAll() {
 		return $this->mounts;
@@ -126,7 +142,7 @@ class Manager {
 	 * Find mounts by numeric storage id
 	 *
 	 * @param int $id
-	 * @return Mount[]
+	 * @return MountPoint[]
 	 */
 	public function findByNumericId($id) {
 		$storageId = \OC\Files\Cache\Storage::getStorageId($id);
